	# below is some code for exploring maxent output in R. at the end of the file are a series of functions needed for the code that should be entered before beginning analysis. the data are also included in the supplement.
	
	#read in the fynbos data (be sure to put the files in your working directory or that you specify the path)
abundance=as.matrix(read.csv(file="fynbos abundance matrix.csv",header=T,row.names=1))
trait.matrix=as.matrix(read.csv(file="fynbos trait matrix.csv",header=T,row.names=1))
number.species=nrow(abundance)
	#traits should be rescaled to the interval [0,1] so that the lagrange multipliers associated with each trait (analogous to regression coefficients) are comparable. note that the Maxent function gives an error if the CAT (community-aggregated trait = abundance weighted trait average in a single community) is 0. this might occur if you have a binary trait where all species in the community have the value 0. and maxent also doesn't like traits that have no variance in a local community. this might arise if you have a trait that varies across a group of communities but that happens to have the same value for all species in one single community.
	#rescaling function
trait.matrix=scale01(trait.matrix)
number.traits=ncol(trait.matrix) 
	#create observed community aggregated traits (CATs)
obs.com.trait=data.frame(t(t(trait.matrix)%*%abundance))
obs.com.trait[obs.com.trait==0]=1e-5 #arbitrary value to make sure no CATs are 0
number.com=nrow(obs.com.trait)
	#values for the environmental gradient (an elevational gradient in this example, with 1 corresponding to the bottom of the mountain and 8 corresponding to the top)
gradient=1:8
	#run EM. note that the priors here are uniform, and are throughout unless otherwise specified
pred=apply(obs.com.trait,1,function(x) Maxent(as.numeric(x),t(trait.matrix))$probabilities)	
	#calcualate the fit
hellinger(pred,abundance) 
	#plot the predicitons for each community seperately
par(mar=c(2,2,1,2), oma=c(4,4,.2,.2),mfrow=c(4,2))
site.plot(pred,abundance,number.species,number.traits,title=' ',gradient) #same as fig.S12
	#plot the predictions for all communities together
cumulative.plot(pred,abundance,number.com) #same as fig 2g
	#get lagrange multipliers
		#the first lagrange multiplier is a constant that corresponds to the normalization constraint
		#this produces a matrix where the lagrage multipliers for a community are given in a row
		#larger values mean more important traits
lagrange.multipliers=matrix(0,number.com,(number.traits+1),dimnames=list(row=paste('site',1:number.com),col=c('constant',colnames(trait.matrix))))
for(i in 1:number.com){
   lamdas=coefficients(lm(log(c(pred[,i]))~.,data=data.frame(trait.matrix))) 
   lamdas[is.na(lamdas)]=0
   lagrange.multipliers[i,]=lamdas
}		
lagrange.multipliers
#mean value across all local communities
apply(lagrange.multipliers,2,mean) #same as Tregional column in table S5 
###################################################
###################################################
	#ALTERNATE MODELS AND EXPLORATION
###################################################
###################################################
#------------------------------------------------------------------------------------
#use predicted CATs instead of observed CATs
library(assist) #contains function 'ssr' used for smoothing splines
	#using predicted CATs allows you to test model predictions, whereas using the observed CATs only allows you to examine model fitting
	#the 'limnla' parameter in the ssr function below controls the smoothness of the spline.
	#note that the pmin and pmax functions used with the predict functions ensure that the predicted CATs remain on the interval (0,1]. maxent makes an error if a CAT = 0, so we artificailly bump it up a little bit (to 1e-6) to some arbitrary value.
spline.com.trait=0*obs.com.trait
for(i in 1:number.traits){
	a=ssr(obs.com.trait[,i]~gradient, rk=cubic2(gradient),limnla=.1)
	spline.com.trait[,i]=pmin(pmax(1e-5,predict(a,data.frame(x=gradient))$fit),1)
}
	#run EM
pred=apply(spline.com.trait,1,function(x) Maxent(as.numeric(x),t(trait.matrix))$probabilities)	
	#plot results
cumulative.plot(pred,abundance,number.com) #same as fig 2h
#------------------------------------------------------------------------------------
#cross validation
		#sequentially withold each local community from the spline fit to CATs along the gradient and predict the CATs at the witheld site. 
	#run EM
pred=0*abundance
for(i in 1:number.com){
	fitting.bins=(1:number.com)[-i]
	for (tr in 1:number.traits){
		t1=ssr(obs.com.trait[fitting.bins,tr]~ fitting.bins,rk=cubic2(fitting.bins), limnla=.1)
		spline.com.trait[,tr]= pmax(1e-5,predict(t1,data.frame(fitting.bins=1:number.com),pstd=FALSE)$fit)
	}
	a=Maxent(as.numeric(spline.com.trait[i,]),t( trait.matrix))
	pred[,i]=a$probabilities
}
	#plot results
cumulative.plot(pred,abundance,number.com) 
#------------------------------------------------------------------------------------
#use informative priors
		#a 'subregional prior' uses the abundance distribution of nearby communities on the gradient as the prior expectation for abundance. in the example below, 2 sites on either side of the focal site are used as a prior (e.g. for the 5th community along the gradient, communities 3-7 are combined to make an abundance distribution to use as the prior
		#note that this assumes that the CATs for each community are organized along your gradient. e.g. if you use a successional gradient, the first column of the CAT matrix is for your youngest community and the last column for the oldest community.
	#get CAT splines
spline.com.trait=0*obs.com.trait
for(i in 1:number.traits){
	a=ssr(obs.com.trait[,i]~gradient, rk=cubic2(gradient),limnla=.1)
	spline.com.trait[,i]=pmin(pmax(1e-5,predict(a,data.frame(x=gradient))$fit),1)
}
	#run EM
pred=0*abundance
for(i in 1:number.com){
	inits=pmax(1e-6,apply(abundance[,max(1,i-2):min(8,i+2)],1,mean))
	a=Maxent(as.numeric(spline.com.trait[i,]),t( trait.matrix),initial.probs=inits)
	pred[,i]=a$probabilities
}	
	#plot results
cumulative.plot(pred,abundance,number.com) #same as fig 2j

		#a 'regional prior' uses the abundance distribution of all communities combined as the prior expectation for abundance
	#run EM
pred=0*abundance
for(i in 1:number.com){
	inits=apply(abundance,1,mean)
	a=Maxent(as.numeric(spline.com.trait[i,]),t( trait.matrix),initial.probs=inits)
	pred[,i]=a$probabilities
}	
	#plot results
cumulative.plot(pred,abundance,number.com)  #same as fig 2i
#------------------------------------------------------------------------------------
#plot the CATs along splines 
layout(matrix(1:number.traits,nrow=2))
for(i in 1:number.traits){
	plot(gradient,obs.com.trait[,i],main=names(obs.com.trait)[i],xlab="succession age",ylab="trait value",pch=19,col='grey50',bty='n')
	lines(gradient,spline.com.trait[,i],lwd=3)
}	
#------------------------------------------------------------------------------------
#significance test of abundance predictions
true.trait.matrix=trait.matrix 
	#set up vectors to record results
reps=20 # this test can be a little slow for a large number of reps
h=rep(NA,reps) 
perm.pred=array(NA,c(number.com,number.species,reps))
trait.spline=FALSE
for(j in 1:reps){
	#permute the trait matrix
    mod.trait.matrix=true.trait.matrix[sample(1 :nrow(true.trait.matrix),nrow(true.trait.matrix),replace=F),]
    mod.com.trait=obs.com.trait
		#option to make the predicted CATs from the splines
	if(trait.spline){ 
	   for(i in 1:number.traits){
		   a=ssr(mod.com.trait[,i]~gradient, rk=cubic2(gradient),limnla=1.5)
		   mod.com.trait[,i]=pmin(pmax(0,predict(a,data.frame(x=gradient))$fit),1)
	   }
	}
	pred=apply(mod.com.trait,1,function(x) Maxent(as.numeric(x),t(mod.trait.matrix))$probabilities)	
	h[j]=hellinger(pred,abundance)
	perm.pred[,,j]=pred
}
	#display results
quantile(h,c(.005,.5,.995))
	#get real predictions to compare to permutations
pred=apply(obs.com.trait,1,function(x) Maxent(as.numeric(x),t(trait.matrix))$probabilities)	
ecdf(h)(hellinger(pred,abundance))

###################################################
###################################################
#FUNCTIONS BELOW
###################################################
###################################################

	#a metric of assessing model fit
hellinger=function(pred,obs,trans=1,dig=3){
	#trans is an exponent that allows you to transform the data, e.g square root
	pred=pred^trans/sum(pred^trans)
	obs=obs^trans/sum(obs^trans)
	1-round(  sqrt(.5*sum( ( sqrt(obs)- sqrt(pred))^2 )) ,dig)
}

	#function to rescale a trait matrix to [0,1]
scale01=function(com.mat){
	for( i in 1:ncol(com.mat)){
		com.mat[,i]=(com.mat[,i]-min(com.mat[,i]))/ (max(com.mat[,i])-min(com.mat[,i])) 
		if(is.na(var(com.mat[,i]))){com.mat[,i]=1}
	}
	com.mat
}
	#shipley's maxent function
Maxent<- function(constraint.means, constraint.matrix, fit=1e-10, initial.probs = 
 NA){
 # MAxent function in R. Written by Bill Shipley
 # Bill.Shipley@USherbrooke.ca
 # Improved Iterative Scaling Algorithm from Della Pietra, S.
 # Della Pietra, , Lafferty, J.  (1997). Inducing features of random fields.
 # IEEE Transactions Pattern Analysis And Machine Intelligence
 # 19:1-13
 #.............................
 # constraint.means is a vector of the community-aggregated trait values
 # constraint.matrix is a matrix with each trait defining a row and each
 # species defining a column.  Thus, two traits and 6 species would produce
 # a constraint.matrix with two rows (the two traits) and 6 columns (the
 # six species)
 if(!is.vector(constraint.means)) return(
 "constraint.means must be a vector")
 if(is.vector(constraint.matrix)) {
	constraint.matrix <- matrix(constraint.matrix,
	nrow = 1, ncol = length(
	constraint.matrix))
 }
 n.constraints <- length(constraint.means)
 dim.matrix <- dim(constraint.matrix)
 if(dim.matrix[2] == 1 & dim.matrix[1] > 1) {
	constraint.matrix <- t(constraint.matrix)
	dim.matrix <- dim(constraint.matrix)
 }
 if(n.constraints != dim.matrix[1]){ return("Number of constraint means not equal to number in constraint.matrix") }
 n.species <- dim.matrix[2]
 if(is.na(initial.probs[1])) { prob <- rep(1/n.species, n.species) }
 if(!is.na(initial.probs[1])) { prob <- initial.probs }
 # C.values is a vector containing the sum of each trait value over the species 
 C.values <- apply(constraint.matrix, 1, sum)
 test1 <- test2 <- 1
 while(test1 > fit & test2 > fit) {
 denom <- constraint.matrix %*% prob
 gamma.values <- log(constraint.means/denom)/
 C.values
 if(sum(is.na(gamma.values)) > 0){ return("missing values in gamma.values") }
 unstandardized <- exp(t(gamma.values) %*% 
 constraint.matrix) * prob
 new.prob <- as.vector(unstandardized/sum(
 unstandardized))
 if(sum(is.na(new.prob)) > 0)
	return("missing values in new.prob")
	test1 <- 100 * sum((prob - new.prob)^2)
	test2 <- sum(abs(gamma.values))
	prob <- new.prob
 }
 list(probabilities = prob, final.moments = denom, constraint.means = constraint.means, 
 	constraint.matrix = constraint.matrix, entropy = -1 * sum(prob * log(prob)))
 }
################# End Shipley et al.'s function ###############

	#plot each local community seperately
site.plot=function(pred,obs,number.species,number.traits,title='',gradient){
	#the grey dots on the left side of the plot are observed zeros that have been jittered so you can see how many there are
	obs.plot=obs;pred.plot=pred
	obs.plot[obs==0]=8e-4
	pred.plot[pred<8e-4]=8e-4
	lim=1
	for(i in 1:(ncol(pred))){
		plot(obs.plot[,i],pred.plot[,i],log='xy',xlim=c(6e-4,1),ylim=c(6e-4,1), xlab=" ",ylab=" ",pch=19,cex.lab=1.5,cex.main=2, cex.axis=1.3,cex=.5,col='white',las=1,xaxt='n',yaxt='n',bty='n')
		text(.02*lim,.7*lim,paste('Elevation Bin = ',gradient[i]),cex=1.2)
		text(0.02*lim,.2*lim,paste("h=",hellinger(obs.plot[,i],pred.plot[,i]),sep=''),font=3,cex=1.1);
		text(.004*lim,.001*lim,expression(rho),cex=1)
		text(.02*lim,.001*lim,paste('=',round(cor(c(rank(obs.plot[,i])), c(rank(pred.plot[,i])),method='p'),3),sep=' '),font=3,cex=1.1)
		abline(0,1,col="grey")
		points(jitter(c(obs.plot[obs.plot[,i]==8e-4,i]),10),c(pred.plot[obs.plot[,i]==8e-4,i]), pch=19,cex=1,col='grey45')
		points(obs.plot[obs.plot[,i]>8e-4,i],pred.plot[obs.plot[,i]>8e-4,i],pch=19,cex=1)
    	axis(1,labels=T,cex.axis=.9,at=c(.001,.01,.1,1),las=1)
    	axis(2,labels=T,cex.axis=.9,at=c(.001,.01,.1,1),las=1)
		}
	mtext('observed abundance',1,outer=T,line=2,cex=1.6)
	mtext('predicted abundance',2,outer=T,line=2,cex=1.6)
}

	# plot of all sites together
cumulative.plot=function(pred,obs,number.alt.bins,histogram=FALSE,lim=NA,subtitle=' '){
	#the grey dots on the left side of the plot are observed zeros that have been jittered so you can see how many there are
	par(mfrow=c(1,1))
	if(is.na(lim)){ lim=max(c(obs,pred)) }
	obs.plot=obs;pred.plot=pred
	obs.plot[obs==0]=8e-4
	pred.plot[pred<8e-4]=8e-4
	plot(c(obs.plot),c(pred.plot),log='xy',xlim=c(6e-4,1),ylim=c(6e-4,1), xlab=" ",ylab=" ",pch=19,cex.lab=1.5,cex.main=2, cex.axis=1.3,cex=.5,col='white',las=1,xaxt='n',yaxt='n',sub=subtitle,bty='n')
#	abline(lm(c(pred)~c(obs)),lwd=2)
	text(.2*lim,.001*lim,expression(rho),cex=1.4)
	text(.4*lim,.001*lim,paste('=',round(cor(c(rank(obs)), c(rank(pred)),method='p'),3),sep=' '),font=3,cex=1.3)	
	text(.3*lim,.004*lim,paste('h =',round(hellinger(c(obs), c(pred)),3),sep=' '),font=3,cex=1.3)
	if(histogram){
		subplot(hist(c(pred[obs==0]),main=' ',ylab=' ',xlab=' '),.3*lim,.85*lim,size=c(1.6,1.6))
		text(.3*lim,.58*lim,'predicted abundance')
		text(.03*lim,.8*lim,'frequency',srt=90)
	}
	abline(0,1,col="grey",lwd=2)
	points(jitter(c(obs.plot[obs.plot==8e-4]),15),c(pred.plot[obs.plot==8e-4]),pch=19,cex=.5,col='grey45')
	points(c(obs.plot[obs.plot>8e-4]),c(pred.plot[obs.plot>8e-4]),pch=19,cex=.5)
    axis(1,labels=F,cex.axis=1.2,at=c(.001,.01,.1,1),las=1)
    axis(1,labels=c('0.001','0.01','0.1','1'),cex.axis=1.2,at=c(.001,.01,.1,1),las=1)
	axis(2,labels=c('0.001','0.01','0.1','1'),cex.axis=1.2,at=c(.001,.01,.1,1),las=1)
	mtext('observed abundance',1,outer=T,line=2,cex=1.6)
	mtext('predicted abundance',2,outer=T,line=2,cex=1.6)
}


